@@ -1,97 +1,24 @@
 #!/usr/bin/perl -w
 #
-# $Id: parted2fdisk.pl 3143 2013-06-14 04:06:09Z bruno $
+# $Id: parted2fdisk.pl 725 2006-07-29 22:46:10Z bruno $
 #
 # parted2fdisk: fdisk like interface for parted 
 # [developped for mindi/mondo http://www.mondorescue.org]
 #
 # Aims to be architecture independant (i386/ia64)
-# Tested on ia64 with RHAS 2.1 - Mandrake 9.0 - RHEL 3.0 - SLES 10
+# Tested on RHAS 2.1 ia64 - Mandrake 9.0 ia64 - RHAS 3.0 ia64
 #
-# Copyright B. Cornec 2000-2013
-# Provided under the GPL v2
+# (c) Bruno Cornec <Bruno.Cornec@hp.com>
+# Licensed under the GPL
 
 use strict;
 
-
-=pod
-
-=head1 NAME
-
-parted2fdisk is a fdisk lie command using parted internally.
-
-=head1 DESCRIPTION
-
-parted2fdisk behaves like a fdisk command, but dialog internally with parted in order to manipulate partiion tables, which allow it to support GPT partition format as well as MBR, contrary to fdisk. It aimed at providing compatible external interface wti fdisk. Developed initialy for ia64 Linux, but is also useful now on x86 systems using GPT partition format (for large HDDs).
-
-=head1 SYNOPSIS
-
-parted2fdisk -s partition
-parted2fdisk -l device
-parted2fdisk [-n] device
-
-=head1 OPTIONS
-
-=over 4
-
-=item B<-s>
-
-Print the size (in blocks) of the given partition.
-
-=item B<-n>
-
-Fake mode. Doesn't pass the commands just simulate.
-
-=item B<-l>
-
-List the partition tables for the specified device and then exit.
-
-=item B<no option>
-
-Allow the creation and manipulation of partition tables.
-
-=back
-
-=head1 ARGUMENTS
-
-=over 4
-
-=item B<partition> 
-
-partition device file (only used with -s option).
-
-=item B<device> 
-
-device file to work on.
-
-=back
-
-=head1 WEB SITES
-
-The main Web site of the project is available at L<http://www.mondorescue.org>. Bug reports should be filled using the trac instance of the project at L<http://trac.mondorescue.org/>.
-
-=head1 USER MAILING LIST
-
-For community exchanges around MondoRescue please use the list L<http://sourceforge.net/mailarchive/forum.php?forum_name=mondo-devel>
-
-=head1 AUTHORS
-
-The MondoRescue team lead by Bruno Cornec L<mailto:bruno@mondorescue.org>.
-
-=head1 COPYRIGHT
-
-MondoRescue is distributed under the GPL v2.0 license or later,
-described in the file C<COPYING> included with the distribution.
-
-=cut
-
-
 $ENV{LANG} = "C";
 $ENV{LANGUAGE} = "C";
 $ENV{LC_ALL} = "C";
 
 # Log 
-my $flog = "/var/log/parted2fdisk.log";
+my $flog = "/tmp/parted2fdisk.log";
 open(FLOG, "> $flog") || die "Unable to open $flog";
 
 my $fdisk = "/sbin/fdisk";
@@ -108,12 +35,9 @@ my $cylend;
 my %start;
 my %end;
 my %type;
+my %flags;
 my $arch;
 my $fake = 0;
-my $mega = 1048576;
-
-# Immediate flushing to avoids read error from mondorestore in log files
-$| = 1;
 
 # Determine on which arch we're running
 if (defined ($ENV{ARCH})) {
@@ -149,11 +73,8 @@ my %pid = (	"FAT" => "6",
 		"fat16" => "e",
 		"ext2" => "83",
 		"ext3" => "83",
-		"ext4" => "83",
-		"xfs" => "83",
-		"reiserfs" => "83",
 		"linux-swap" => "82",
-		"lvm" => "8e",
+		"LVM" => "8e",
 		"" => "",
 	);
 my %pnum;
@@ -217,8 +138,8 @@ if ($arch =~ /^ia64/) {
 			while ($i = <STDIN>) {
 				if ($i =~ /^p$/) {
 					fdisk_list($device,undef,\%start,\%end, 1);
-					print "command (m for help) send back to fake fdisk for mondorestore\n";
-				} elsif ($i =~ /^n$/) {
+				}
+				elsif ($i =~ /^n$/) {
 					fdisk_list($device,undef,\%start,\%end, 0);
 					if ($type ne "gpt") {
 						print FLOG "Forcing GPT type of disk label\n";
@@ -243,7 +164,7 @@ if ($arch =~ /^ia64/) {
 					if ((not (defined $cylstart)) || ($cylstart eq "")) {
 						if (defined $start{$part-1}) {
 							# in MB => cyl
-							$cylstart = sprintf("%d",$end{$part-1}*$mega/$un + 1);
+							$cylstart = sprintf("%d",$end{$part-1}*1048576/$un + 1);
 							print FLOG "no start cyl given for creation... assuming the following $cylstart\n";
 						} else {
 							print FLOG "no start cyl given for creation... assuming the following 1\n";
@@ -257,7 +178,7 @@ if ($arch =~ /^ia64/) {
 					if ($cylstart == 1) {
 						$start = 0.01;
 					} else {
-						$start = $cylstart* $un / $mega + 0.001;
+						$start = $cylstart* $un / 1048576 +0.001;
 					}
 					# this is a size in B/KB/MB/GB
 
@@ -272,8 +193,16 @@ if ($arch =~ /^ia64/) {
 					# to give cylinders
 					if ($cylend =~ /^\+/) {
 						$cylend =~ s/^\+//;
-						# Handles suffixes; return bytes
-						$cylend = decode_Bsuf($cylend,1);
+						if ($cylend =~ /K$/) {
+							$cylend =~ s/K$//;
+							$cylend *= 1024;
+						} elsif ($cylend =~ /M$/) {
+							$cylend =~ s/M$//;
+							$cylend *= 1048576;
+						} elsif ($cylend =~ /G$/) {
+							$cylend =~ s/G$//;
+							$cylend *= 1073741824;
+						}
 						# This gives the number of cyl
 						$cylend /= $un;
 						$cylend = sprintf("%d",$cylend);
@@ -283,11 +212,11 @@ if ($arch =~ /^ia64/) {
 					$cylend = $endmax if ($cylend > $endmax); 
 					print FLOG "end cyl : $cylend\n";
 					# parted needs MB
-					$end = $cylend * $un / $mega;
+					$end = $cylend * $un / 1048576;
 					print FLOG "n $l $part $cylstart $cylend => mkpart primary $start $end\n";
 					system "$parted -s $device mkpart primary ext2 $start $end\n" if ($fake == 0);
-					print "command (m for help) send back to fake fdisk for mondorestore\n";
-				} elsif ($i =~ /^d$/) {
+				}
+				elsif ($i =~ /^d$/) {
 					$part = <STDIN>;
 					if (not (defined $part)) {
 						print FLOG "no partition given for deletion... skipping\n";
@@ -297,23 +226,18 @@ if ($arch =~ /^ia64/) {
 					print FLOG "d $part => rm $part\n";
 					system "$parted -s $device rm $part\n" if ($fake == 0);
 					get_parted($device,undef,\%start,\%end,undef);
-					print "command (m for help) send back to fake fdisk for mondorestore\n";
-				} elsif ($i =~ /^w$/) {
+				}
+				elsif ($i =~ /^w$/) {
 					print FLOG "w => quit\n";
-				} elsif ($i =~ /^t$/) {
+				}
+				elsif ($i =~ /^t$/) {
 					$part = <STDIN>;
 					if (not (defined $part)) {
 						print FLOG "no partition given for tagging... skipping\n";
 						next;
 					}
 					chomp($part);
-					# If no partition number given it's 1, and we received the type
-					if ($part !~ /\d+/) {
-						$l = $part;
-						$part = 1 
-					} else {
-						$l = <STDIN>;
-					}
+					$l = <STDIN>;
 					if (not (defined $l)) {
 						print FLOG "no type given for tagging partition $part... skipping\n";
 						next;
@@ -323,35 +247,24 @@ if ($arch =~ /^ia64/) {
 						print FLOG "no partition number given for $l... please report to the author\n";
 						next;
 					}
-
-					if ($pnum{$l} eq "lvm") {
-						# In that case this is a flag set, not a mkfs
-						print FLOG "t $part $l => set $part $pnum{$l} on\n";
-						system "$parted -s $device set $part $pnum{$l} on\n" if ($fake == 0);
-					} else {
-						print FLOG "t $part $l => mkfs $part $pnum{$l}\n";
-						system "$parted -s $device mkfs $part $pnum{$l}\n" if ($fake == 0);
-					}
-					print "command (m for help) send back to fake fdisk for mondorestore\n";
-				} elsif ($i =~ /^a$/) {
+					print FLOG "t $part $l => mkfs $part $pnum{$l}\n";
+					system "$parted -s $device mkfs $part $pnum{$l}\n" if ($fake == 0);
+				}
+				elsif ($i =~ /^a$/) {
 					$part = <STDIN>;
 					if (not (defined $part)) {
 						print FLOG "no partition given for tagging... skipping\n";
 						next;
 					}
 					chomp($part);
-
-					# Partition shouldn't be negative or null. Then take the first one.
-					$part = 1 if ($part le 0);
-
 					print FLOG "a $part => set $part boot on\n";
 					system "$parted -s $device set $part boot on\n" if ($fake == 0);
-					print "command (m for help) send back to fake fdisk for mondorestore\n";
-				} elsif ($i =~ /^q$/) {
+				}
+				elsif ($i =~ /^q$/) {
 					print FLOG "q => quit\n";
-				} else {
+				}
+				else {
 					print FLOG "Unknown command: $i\n";
-					print "command (m for help) send back to fake fdisk for mondorestore\n";
 					next;
 				}
 					
@@ -370,14 +283,14 @@ my $fargs = join(@ARGV);
 
 if ($args =~ /^-/) {
 	# -l or -s
-	open (FDISK, "$fdisk $fargs 2>/dev/null |") || die "Unable to read from $fdisk";
+	open (FDISK, "$fdisk $fargs |") || die "Unable to read from $fdisk";
 	while (<FDISK>) {
 		print;
 	}
 	close(FDISK);
 } else {
 	# Modification mode
-	open (FDISK, "| $fdisk $fargs 2>/dev/null") || die "Unable to modify through $fdisk";
+	open (FDISK, "| $fdisk $fargs") || die "Unable to modify through $fdisk";
 	while (<STDIN>) {
 		print FDISK;
 	}
@@ -428,11 +341,8 @@ my $n;
 my %cmt = (	"FAT" => "FAT",
 		"ext2" => "Linux",
 		"ext3" => "Linux",
-		"ext4" => "Linux",
-		"xfs" => "Linux",
-		"reiserfs" => "Linux",
 		"linux-swap" => "Linux swap",
-		"lvm" => "Linux LVM",
+		"LVM" => "Linux LVM",
 		"fat16" => "fat16",
 		"fat32" => "fat32",
 		"" => "Linux",
@@ -445,28 +355,26 @@ my $length;
 my $pid;
 my $cmt;
 format FLOG1 =
-@<<<<<<<<<<<< @>>>>>>>>>> @>>>>>>>>>> @>>>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+@<<<<<<<<<<<< @>>>>>>>> @>>>>>>>> @>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $part,        $mstart,   $mend,   $length,  $pid, $cmt
 .
 format FLOG2 =
 @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $part,
-              @>>>>>>>>>> @>>>>>>>>>> @>>>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+              @>>>>>>>> @>>>>>>>> @>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
               $mstart,   $mend,   $length,  $pid, $cmt
 .
 format STDOUT1 =
-@<<<<<<<<<<<< @>>>>>>>>>> @>>>>>>>>>> @>>>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+@<<<<<<<<<<<< @>>>>>>>> @>>>>>>>> @>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $part,        $mstart,   $mend,   $length,  $pid, $cmt
 .
 format STDOUT2 =
 @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 $part,
-              @>>>>>>>>>> @>>>>>>>>>> @>>>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
+              @>>>>>>>> @>>>>>>>> @>>>>>>>> @>>>  @<<<<<<<<<<<<<<<<<<<<<<<<<<<<
               $mstart,   $mend,   $length,  $pid, $cmt
 .
-#   Device Boot      Start         End      Blocks   Id  System
-#/dev/hda1               1       77579    39099374+  ee  EFI GPT
-
+#/dev/hda1             1     77579  39099374+  ee  EFI GPT
 
 #
 # Keep Fdisk headers
@@ -477,19 +385,17 @@ $un = get_un ($device,$wpart,$verbose);
 $endmax = get_max($device);
 
 # This will return MB
-get_parted ($device,$start,$end,\%type);
+get_parted ($device,$start,$end,\%type,\%flags);
 
 while (($n,$d) = each %type) {
 	# Print infos fdisk like
 	$part = ${device}.$n;
 	# start and end are in cylinder in fdisk format
 	# so return in MB * 1MB / what represents 1 cyl in B
-	$mstart = sprintf("%d",$$start{$n}*$mega/$un);
+	$mstart = sprintf("%d",$$start{$n}*1048576/$un);
 	$mstart = 1 if ($mstart < 1);
-	$mstart = $endmax if ($mstart > $endmax);
-	$mend = sprintf("%d",$$end{$n}*$mega/$un - 1);
+	$mend = sprintf("%d",$$end{$n}*1048576/$un - 1);
 	$mend = $endmax if ($mend > $endmax);
-	$mend = 1 if ($mend < 1);
 	# length is in 1K blocks
 	$length = sprintf("%d",($mend-$mstart+1)*$un/1024);
 	$pid = $pid{$type{$n}};
@@ -539,7 +445,7 @@ my $device = shift;
 my $max = 0;
 my $foo;
 
-open (FDISK, "$fdisk -l $device 2>/dev/null |") || die "Unable to read from $fdisk";
+open (FDISK, "$fdisk -l $device |") || die "Unable to read from $fdisk";
 while (<FDISK>) {
 	if ($_ =~ /heads/) {
 		chomp;
@@ -563,7 +469,7 @@ my $verbose = shift;
 my $un = 0;
 my $foo;
 
-open (FDISK, "$fdisk -l $device 2>/dev/null |") || die "Unable to read from $fdisk";
+open (FDISK, "$fdisk -l $device |") || die "Unable to read from $fdisk";
 while (<FDISK>) {
 	print if (($_ !~ /^\/dev\//) and (not (defined $wpart)) and ($verbose == 1));
 	if ($_ =~ /^Units/) {
@@ -579,7 +485,6 @@ return($un);
 
 # 
 # Parted gives info in MB
-# (depending on versions - 1.6.25.1 provides suffixes)
 #
 sub get_parted {
 
@@ -587,93 +492,33 @@ my $device = shift;
 my $start = shift;
 my $end = shift;
 my $type = shift;
-my $void;
+my $flags = shift;
 my $d;
 my $n;
-my $ret;
-my $mode;
-my $size;
-my $unit;
-
-open (PARTED, "$parted -v |") || die "Unable to read from $parted";
-$d = <PARTED>;
-print FLOG "$d";
-close(PARTED);
 
 open (PARTED, "$parted -s $device print |") || die "Unable to read from $parted";
 # Skip 3 first lines
 $d = <PARTED>;
 $d = <PARTED>;
 $d = <PARTED>;
-
-# depending on parted version, information given change:
-if ($d =~ /\bSize\b/) {
-	# SLES 10 parted >= 1.6.25
-	$mode=1;
-} else {
-	# RHEL 3 parted 1.6.3
-	# RHEL 4 parted 1.6.19
-	$mode=0;
-}
-print FLOG "mode: $mode\n";
 print FLOG "Got from parted: \n";
-print FLOG "Minor    Start       End     Filesystem\n";
+print FLOG "Minor    Start       End     Filesystem                        Flags\n";
 # Get info from each partition line
 while (($n,$d) = split(/\s/, <PARTED>,2)) {
 	chomp($d);
 	next if ($n !~ /^[1-9]/);
 	$d =~ s/^\s*//;
 	$d =~ s/\s+/ /g;
-	if ($mode == 0) {
-		($$start{$n},$$end{$n},$$type{$n},$void) = split(/ /,$d);
-		$unit = 1;
-	} elsif ($mode == 1) {
-		($$start{$n},$$end{$n},$size,$$type{$n},$void) = split(/ /,$d);
-		$unit = $mega;
-	} else {
-		die "Undefined mode $mode";
-	}
+	($$start{$n},$$end{$n},$$type{$n},$$flags{$n}) = split(/ /,$d);
 	$$start{$n} = "" if (not defined $$start{$n});
 	$$end{$n} = "" if (not defined $$end{$n});
 	$$type{$n} = "" if (not defined $$type{$n});
-	# Handles potential suffixes in latest parted version. Return MB
-	$ret = decode_Bsuf($$start{$n},$unit);
-	$$start{$n} = $ret;
-	$ret = decode_Bsuf($$end{$n},$unit);
-	$$end{$n} = $ret;
-	print FLOG "$n      $$start{$n}      $$end{$n}     $$type{$n}\n";
+	$$flags{$n} = "" if (not defined $$flags{$n});
+	print FLOG "$n      $$start{$n}      $$end{$n}     $$type{$n}  $$flags{$n}\n";
 }
 close(PARTED);
 }
 
-sub decode_Bsuf {
-
-my $size  = shift;
-my $unit  = shift;
-my $ret = 0;
-
-#print FLOG "decode_Bsuf input: $size / $unit ";
-if ($size =~ /K[B]*$/i) {
-	$size =~ s/K[B]*$//i;
-	$size *= 1024;
-} elsif ($size =~ /M[B]*$/i) {
-	$size =~ s/M[B]*$//i;
-	$size *= 1048576;
-} elsif ($size =~ /G[B]*$/i) {
-	$size =~ s/G[B]*$//i;
-	$size *= 1073741824;
-} elsif ($size =~ /T[B]*$/i) {
-	$size =~ s/T[B]*$//i;
-	$size *= 1099511627776;
-} else {
-	# Nothing to do
-}
-$ret = $size / $unit;
-#print FLOG " - output : $size => $ret\n";
-return($ret);
-}
-
-
 # Based on Version 2.4  27-Sep-1996  Charles Bailey  bailey@genetics.upenn.edu
 # in Basename.pm
 
@@ -701,7 +546,7 @@ sub which_type {
 my $device = shift;
 my $type = "";
 
-open (FDISK, "$fdisk -l $device 2>/dev/null |") || die "Unable to read from $fdisk";
+open (FDISK, "$fdisk -l $device |") || die "Unable to read from $fdisk";
 while (<FDISK>) {
 	if ($_ =~ /EFI GPT/) {
 		$type= "gpt";
